function z = findq(q,parameters,max_iter,penalty,tol)

beta  = parameters.beta;
alpha = parameters.alpha;

agrid = parameters.agrid; na = numel(agrid);

a     = parameters.a; ns = numel(a);
y     = parameters.y;
P     = parameters.P; 

%%%%% reward function

c    = zeros(ns,na);

for j=1:na,
    
    c(:,j) = a + y - q*agrid(j);
  
end

%%%%% check consumption non-negative

violations = (c<=0);

c = c.*(c>=0) + eps;

if alpha==1,
    
    u = log(c) - penalty*violations;
    
else
    
    u = (1/(1-alpha))*(c.^(1-alpha) - 1) - penalty*violations;

end

%%%%% initialize inner dynamic programming loop

v       = log(0.5*y)/(1-beta);

for i=1:max_iter,

RHS      = u+beta*kron(P,ones(na,1))*reshape(v,na,2)';

[Tv,argmax] = max(RHS,[],2);

%%%%% policy that attains the maximum

g = a(argmax);

%%%%% check if converged

error = max(abs(Tv-v));

%fprintf('%4i %6.2e \n',[i, error]);

if norm(Tv-v,inf)<tol, break, end;

%%%%% if not converged, update guess

v = Tv;

end

%%%%% market clearing condition

%%%%% construct transition matrix for the state s=(a,y)

A = zeros(ns,na);
Q = zeros(ns,ns);

PP = kron(P,ones(na,1));

for s=1:ns,
    
    A(s,:) = (agrid==g(s))';  %% puts a 1 if g(s)=a 
    
    Q(s,:) = kron(PP(s,:),A(s,:));
    
end

%%%%% compute stationary distribution

[eig_vectors,eig_values] = eig(Q'); 
[~,arg] = min(abs(diag(eig_values)-1)); 
unit_eig_vector = eig_vectors(:,arg); 

mu = unit_eig_vector/sum(unit_eig_vector); 

%%%%% check market clearing

z = sum(mu.*g);

fprintf('[q, z=F(q)] %2.7f  %3.4f  \n',[q, z]);

